pub struct InternedString (i64);

#[derive (Default)]
pub struct Interner {
	table: std::collections::BTreeMap <String, i64>,
	counter: i64,
}

impl Interner {
	pub fn intern (&mut self, s: &str) -> InternedString {
		match self.table.get (s) {
			Some (x) => InternedString (*x),
			None => {
				self.counter += 1;
				if self.counter == i64::MAX {
					panic! ("Out of IDs");
				}
				self.table.insert (s.to_string (), self.counter);
				InternedString (self.counter)
			}
		}
	}
}

#[cfg (test)]
mod tests {
	use super::*;
	
	#[test]
	fn test () {
		let mut interner = Interner::default ();
		
		assert_eq! (interner.intern ("x").0, 1);
		assert_eq! (interner.intern ("x").0, 1);
		assert_eq! (interner.intern ("y").0, 2);
		assert_eq! (interner.intern ("z").0, 3);
		assert_eq! (interner.intern ("y").0, 2);
		
		assert_eq! (interner.intern ("asdf").0, 4);
	}
}
